﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: phil
 * Dátum: 2008.08.13.
 * Idő: 10:27
 * Verzió: 0.0.0
 * Legutóbbi változtatás időpontja:
 * Változtatások:
 * Teendők:
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization;
using System.Windows.Forms.DataVisualization.Charting;
//
using TetheredSun.Controls;
using Noise.DataAcquisition;
using TetheredSun.SignalAnalysis;

namespace System.Windows.Forms.DataVisualization
{

	public class MSDataPlotter : Chart, IDataPlotter
	{	
		protected volatile int historyLength = 128;
		protected int dueTime = Timeout.Infinite;
		protected int interval = 100;
		protected double timeIncrement;
		protected bool autoScale = false; // For future use.
		protected bool channelsInitialised = false;
		protected bool levelsInitialised = false;
		protected List<IChannelDataSocket<double>> channels;
		protected System.Threading.Timer timer;
		protected volatile bool isReady = true;
		protected volatile bool stopped = false;
		protected bool isLevelVisible = true;
		protected Color defaultLineColour = Color.Maroon;
		protected int defaultLineWidth = 2;
		protected LevelCrossingDetector levelCrossingDetector = null;
	
		
		public MSDataPlotter(double timeIncrement) : base()
		{
			this.timeIncrement = timeIncrement;
			this.channels = new List<IChannelDataSocket<double>>();
			this.timer = new System.Threading.Timer(new TimerCallback(UpdatePlots), null, Timeout.Infinite, interval);
		}
		
		public MSDataPlotter() : this(0.02) { }
		
		public List<IChannelDataSocket<double>> Channels {
			get { return channels; }
		}
		
		public Color DefaultLineColour {
			get { return defaultLineColour; }
			set { defaultLineColour = value; }
		}
		
		public int DefaultLineWidth {
			get { return defaultLineWidth; }
			set { defaultLineWidth = value; }
		}
				
		public int HistoryLength {
			get { return historyLength; }
			set { historyLength = value; }
		}
		
		public int Interval {
			get { return interval; }
			set {
				interval = value;
				timer.Change(dueTime, this.interval);
			}
		}
		
		public LevelCrossingDetector LevelCrossingDetector {
			get { return levelCrossingDetector; }
			set { levelCrossingDetector = value; }
		}
		
		
		public double TimeIncrement {
			get { return timeIncrement; }
			set { timeIncrement = value; }
		}
		
		
		public void AddChannel(IChannelDataSocket<double> channel, Color lineColour, int lineWidth)
		{
			Series series;
			
			if (!channels.Contains(channel)) {
				channels.Add(channel);
			}
			
			try {
				series = this.Series[channel.ChannelInfo.Name];
			} catch {
				series = new Series(channel.ChannelInfo.Name);
				series.ChartType = SeriesChartType.FastLine;
				this.Series.Add(series);
				LabelYAxis(channel.ChannelInfo);
			}
			
			series.Color = lineColour;
			series.BorderWidth = lineWidth;
		}
		
		public void Reset()
		{
			this.Series.Clear();
			channelsInitialised = false;
			levelsInitialised = false;
		}
	
		public void Start()
		{
			stopped = false;
			dueTime = 0;
			timer.Change(dueTime, interval);
		}
		
		public void Start(int interval)
		{
			this.interval = interval;
			Start();
		}
		
		public void Stop()
		{
			stopped = true;
			dueTime = Timeout.Infinite;
			timer.Change(dueTime, interval);
		}
		
		protected void LabelYAxis(ChannelInfo channelInfo)
		{
			if ((channelInfo.Sensor != null) && (!String.IsNullOrEmpty(channelInfo.Sensor.Quantity))) {
				this.ChartAreas[0].AxisY.Title = channelInfo.Sensor.Quantity;
				if (!String.IsNullOrEmpty(channelInfo.Sensor.Unit)) {
					this.ChartAreas[0].AxisY.Title += String.Format(" [{0}]", channelInfo.Sensor.Unit);
				}
			} else {
				this.ChartAreas[0].AxisY.Title = "U [V]";
			}
		}
			
		
		[EditorBrowsableAttribute()]
		protected override void OnCreateControl()
		{
			base.OnCreateControl();
			this.Series.Clear();
			this.Legends.Clear();
			//
			MSChartFormatter.Format(this);
		}	
		
		protected void UpdatePlots(object state)
		{
			if (!isReady) return;
			if (stopped) return;

			if (this.InvokeRequired) {
				TimerCallback method = UpdatePlots;
				this.Invoke(method, new object[] { state });
			} else {
				isReady = false;
				bool showLevels = (levelCrossingDetector != null) && levelCrossingDetector.IsActive && isLevelVisible; 
				bool showLevel2 = showLevels && (levelCrossingDetector.Hysteresis != 0.0);
				Series current = null;
				Series level1 = null;
				Series level2 = null;
				Series upCrossings = null;
				int itemsInQueue = 0;
				double time;
				double item;
				double[] dataPoint;
				List<double[]> newItems;
				
				
				if (levelsInitialised) {
					level1 = this.Series["Level1"];
					level2 = this.Series["Level2"];
					upCrossings = this.Series["UpCrossings"];
				} else {
					level1 = new Series("Level1");
					level1.ChartType = SeriesChartType.FastLine;
					level1.BorderWidth = 2;
					level1.Color = Color.Orange;
					this.Series.Add(level1);
					//
					level2 = new Series("Level2");
					level2.ChartType = SeriesChartType.FastLine;
					level2.BorderWidth = 2;
					level2.Color = Color.Orange;
					this.Series.Add(level2);
					//
					upCrossings = new Series("UpCrossings");
					upCrossings.ChartType = SeriesChartType.Point;
					upCrossings.BorderWidth = 2;
					upCrossings.MarkerSize = 10;
					upCrossings.MarkerStyle = MarkerStyle.Cross;
					upCrossings.Color = Color.Firebrick;
					this.Series.Add(upCrossings);
					levelsInitialised = true;
				}
				
				if (!showLevels && (level1 != null)) {
					level1.Points.Clear();
				}
				
				if (!showLevels && (level2 != null)) {
					level2.Points.Clear();
				}
				
				if (!showLevels && (upCrossings != null)) {
					upCrossings.Points.Clear();
				}
	
				
				foreach (IChannelDataSocket<double> channel in channels) {
					if (channelsInitialised) {
						current = this.Series[channel.ChannelInfo.Name];
					} else {
						current = new Series(channel.ChannelInfo.Name);
						current.ChartType = SeriesChartType.FastLine;
						current.BorderWidth = defaultLineWidth;
						current.Color = defaultLineColour;
						this.Series.Add(current);
						LabelYAxis(channel.ChannelInfo);
						channelsInitialised = true;
					}
					
					if (current.Points.Count > 0) {
						time = current.Points[current.Points.Count - 1].XValue + timeIncrement;
					} else {
						time = channel.ChannelInfo.TimeOffset;
					}
					
					newItems = new List<double[]>();
					current.Points.SuspendUpdates();
					
					lock (channel.SyncRoot) {
						itemsInQueue = channel.Count;
						for (int i = 0; i < itemsInQueue; i++) {
							item = channel.Dequeue();
							dataPoint = showLevels ? new double[] {
								time,
								item,
								levelCrossingDetector.Level - levelCrossingDetector.Hysteresis,
								levelCrossingDetector.Level + levelCrossingDetector.Hysteresis 
							} : new double[] { time, item };
							newItems.Add(dataPoint);
						
							if (levelCrossingDetector != null) {
								levelCrossingDetector.Test(time, item);
							}
							time += timeIncrement;
						}
					}
					foreach (double[] newItem in newItems) {
						current.Points.AddXY(newItem[0], newItem[1]);
						if (showLevels) {
							level1.Points.AddXY(newItem[0], newItem[2]);
							if (showLevel2) level2.Points.AddXY(newItem[0], newItem[3]);
						}
						while (current.Points.Count > historyLength) {
							current.Points.RemoveAt(0);
						}
						if (level1 != null) {
							while (level1.Points.Count > historyLength) {
								level1.Points.RemoveAt(0);
							}
						}
						if (level2 != null) {
							while (level2.Points.Count > historyLength) {
								level2.Points.RemoveAt(0);
							}
						}
					}
					
					if (showLevels) {
						upCrossings.Points.Clear();
						for (int i = 0; i < levelCrossingDetector.Cycles.Count; i++) {
							double t = levelCrossingDetector.Cycles[i].UpCrossing;
							if (t >= current.Points[0].XValue) {
								upCrossings.Points.AddXY(t, levelCrossingDetector.Level);
							}
						}
					}

					current.Points.ResumeUpdates();
				}
				this.ChartAreas[0].RecalculateAxesScale();
				this.Refresh();
				isReady = true;
			}
		}
		
	}
}
