﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: phil
 * Dátum: 2008.08.13.
 * Idő: 10:27
 * Verzió: 0.0.0
 * Legutóbbi változtatás időpontja:
 * Változtatások:
 * Teendők:
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization;
using System.Windows.Forms.DataVisualization.Charting;
//
using TetheredSun.Controls;
using TetheredSun.Core;
using Noise.DataAcquisition;
using TetheredSun.SignalAnalysis;

namespace System.Windows.Forms.DataVisualization
{

	public class MSDataPlotter : Chart
	{	
		protected volatile int historyLength = 128;
		protected int dueTime = Timeout.Infinite;
		protected int interval = 100;
		protected double time = 0.0;
		protected double timeIncrement;
		protected bool autoScale = false; // For future use.
		protected bool channelsInitialised = false;
		protected bool levelsInitialised = false;
		protected IChannelDataSocket<double> channel;
		protected System.Threading.Timer timer;
		protected volatile bool isReady = true;
		protected volatile bool stopped = false;
		protected bool isLevelVisible = true;
		protected Color defaultLineColour = Color.Maroon;
		protected int defaultLineWidth = 2;
		protected LevelCrossingDetector levelCrossingDetector = null;
		protected Series data;
		protected Series level1;
		protected Series level2;
		protected Series upCrossings;
		protected Series downCrossings;
		protected RingBuffer<double[]> dataBuffer;
		protected RingBuffer<Point<double>> upBuffer;
		protected RingBuffer<Point<double>> downBuffer;
	
		
		public MSDataPlotter(double timeIncrement) : base()
		{
			this.timeIncrement = timeIncrement;
			this.timer = new System.Threading.Timer(new TimerCallback(Update), null, Timeout.Infinite, interval);
			//
			dataBuffer = new RingBuffer<double[]>(historyLength);
			upBuffer = new RingBuffer<Point<double>>(32);
			downBuffer = new RingBuffer<Point<double>>(32);
		}
		
		public MSDataPlotter() : this(0.02) { }
		
		public IChannelDataSocket<double> Channel {
			get { return channel; }
			set { 
				channel = value;
				if (channel != null) LabelYAxis(channel.ChannelInfo);
			}
		}
		
		public Color DefaultLineColour {
			get { return defaultLineColour; }
			set { defaultLineColour = value; }
		}
		
		public int DefaultLineWidth {
			get { return defaultLineWidth; }
			set { defaultLineWidth = value; }
		}
				
		public int HistoryLength {
			get { return historyLength; }
			set {
				historyLength = value;
				lock (dataBuffer.SyncRoot) { dataBuffer.Capacity = historyLength; }
				ResizeSeries();
			}
		}
		
		public int Interval {
			get { return interval; }
			set {
				interval = value;
				timer.Change(dueTime, this.interval);
			}
		}
		
		public LevelCrossingDetector LevelCrossingDetector {
			get { return levelCrossingDetector; }
			set { levelCrossingDetector = value; }
		}
		
		
		public double TimeIncrement {
			get { return timeIncrement; }
			set { timeIncrement = value; }
		}
		
		
		public void CopyToClipBoard()
		{
			string timeLabel = "t [s]";
			string valueLabel;
			string unitLabel;
			string channelString = "Channel ";
			string clipboardString = String.Empty;
			
			if ((channel != null) && (channel.ChannelInfo != null)) {
				if (channel.ChannelInfo.Sensor != null) {
					valueLabel = channel.ChannelInfo.Sensor.GetLabel();
					unitLabel = (String.IsNullOrEmpty(channel.ChannelInfo.Sensor.Unit)) ?
					" [V]" : String.Format(" [{0}]", channel.ChannelInfo.Sensor.Unit);
				} else {
					valueLabel = "U [V]";
					unitLabel = " [V]";
				}
				
				channelString += channel.ChannelInfo.Name;
			} else {
				valueLabel = "U [V]";
				unitLabel = " [V]";
				channelString += "unknown";
			}
			
			clipboardString += channelString + "\t\t\t\t\t\t\t" + Environment.NewLine;
			
			if (data.Points.Count > 0) {
				clipboardString += timeLabel + "\t" + valueLabel + "\t";
				if (level1.Points.Count > 0) {
					clipboardString += "Level 1" + unitLabel + "\t";
				} else {
					clipboardString += "\t";
				}
				if (level2.Points.Count > 0) {
					clipboardString += "Level 2" + unitLabel + "\t";
				} else {
					clipboardString += "\t";
				}
				if (upCrossings.Points.Count > 0) {
					clipboardString += "t up [s]" + "\t" + valueLabel + "\t";
				} else {
					clipboardString += "\t";
				}
				if (downCrossings.Points.Count > 0) {
					clipboardString += "t down [s]" + "\t" + valueLabel + "\t";
				}  else {
					clipboardString += "\t";
				}
				clipboardString = clipboardString.Trim("\t".ToCharArray());
				clipboardString += Environment.NewLine;
			}
			
			for (int i = 0; i < data.Points.Count; i++) {
				clipboardString += data.Points[i].XValue.ToString() + "\t";
				clipboardString += data.Points[i].YValues[0].ToString() + "\t";
				if (i < level1.Points.Count) {
					clipboardString += level1.Points[i].YValues[0].ToString() + "\t";
				} else {
					clipboardString += "\t";
				}
				if (i < level2.Points.Count) {
					clipboardString += level2.Points[i].YValues[0].ToString() + "\t";
				} else {
					
				}
				if (i < upCrossings.Points.Count) {
					clipboardString += upCrossings.Points[i].XValue.ToString() + "\t";
					clipboardString += upCrossings.Points[i].YValues[0].ToString() + "\t";
				} else {
					clipboardString += "\t\t";
				}
				if (i < downCrossings.Points.Count) {
					clipboardString += downCrossings.Points[i].XValue.ToString() + "\t";
					clipboardString += downCrossings.Points[i].YValues[0].ToString() + "\t";
				} else {
					clipboardString += "\t\t";
				}
				clipboardString = clipboardString.Trim("\t".ToCharArray());
				clipboardString += Environment.NewLine;
			}
			Clipboard.SetDataObject(clipboardString);
		}
		
		public void Initialise()
		{
			this.Series.Clear();
			//
			data = new Series("Data");
			data.ChartType = SeriesChartType.FastLine;
			data.BorderWidth = defaultLineWidth;
			this.Series.Add(data);
			//
			level1 = new Series("Level 1");
			level1.ChartType = SeriesChartType.FastLine;
			level1.BorderWidth = 2;
			level1.Color = Color.Orange;
			this.Series.Add(level1);
			//
			level2 = new Series("Level 2");
			level2.ChartType = SeriesChartType.FastLine;
			level2.BorderWidth = 2;
			level2.Color = Color.Orange;
			this.Series.Add(level2);
			//
			upCrossings = new Series("UpCrossings");
			upCrossings.ChartType = SeriesChartType.Point;
			upCrossings.BorderWidth = 2;
			upCrossings.MarkerSize = 15;
			upCrossings.MarkerStyle = MarkerStyle.Cross;
			upCrossings.Color = Color.Firebrick;
			this.Series.Add(upCrossings);
			//
			downCrossings = new Series("DownCrossings");
			downCrossings.ChartType = SeriesChartType.Point;
			downCrossings.BorderWidth = 2;
			downCrossings.MarkerSize = 15;
			downCrossings.MarkerStyle = MarkerStyle.Cross;
			downCrossings.Color = Color.SteelBlue;
			this.Series.Add(downCrossings);
		}
		
		public void Reset()
		{
			time = 0.0;
			lock (dataBuffer.SyncRoot) {
				dataBuffer.Clear();
			}
			lock (upBuffer.SyncRoot) {
				upBuffer.Clear();
			}
			lock (downBuffer.SyncRoot) {
				downBuffer.Clear();
			}
			foreach (Series series in this.Series) {
				series.Points.Clear();
			}
			if (levelCrossingDetector != null) {
				levelCrossingDetector.Cycles.Clear();
			}
		}
	
		public void Start()
		{
			stopped = false;
			dueTime = 0;
			timer.Change(dueTime, interval);
		}
		
		public void Start(int interval)
		{
			this.interval = interval;
			Start();
		}
		
		public void Stop()
		{
			stopped = true;
			dueTime = Timeout.Infinite;
			timer.Change(dueTime, interval);
		}
		
		
		protected void LabelYAxis(ChannelInfo channelInfo)
		{
			if ((channelInfo.Sensor != null) && (!String.IsNullOrEmpty(channelInfo.Sensor.Quantity))) {
				this.ChartAreas[0].AxisY.Title = channelInfo.Sensor.Quantity;
				if (!String.IsNullOrEmpty(channelInfo.Sensor.Unit)) {
					this.ChartAreas[0].AxisY.Title += String.Format(" [{0}]", channelInfo.Sensor.Unit);
				}
			} else {
				this.ChartAreas[0].AxisY.Title = "U [V]";
			}
		}
			
		
		[EditorBrowsableAttribute()]
		protected override void OnCreateControl()
		{
			base.OnCreateControl();
			this.Legends.Clear();
			//
			MSChartFormatter.Format(this);
		}
		
		protected void ResizeSeries()
		{
			foreach (Series series in this.Series) {
				series.Points.SuspendUpdates();
				while (series.Points.Count > historyLength) {
					series.Points.RemoveAt(0);
				}
				series.Points.ResumeUpdates();
			}
		}
		
		protected void Update(object state)
		{
			int itemsInQueue;
			double[] dataPoint;
			double value;
			double level1 = (levelCrossingDetector != null) ? levelCrossingDetector.Level - levelCrossingDetector.Hysteresis :
				Double.NaN;
			double level2 = (levelCrossingDetector != null) ? levelCrossingDetector.Level + levelCrossingDetector.Hysteresis :
				Double.NaN;
			Point<double> upCrossing = null;
			Point<double> downCrossing = null;
			
			lock (channel.SyncRoot) {
				itemsInQueue = channel.Count;
				for (int i = 0; i < itemsInQueue; i++) {
					value = channel.Dequeue();
					
					if (levelCrossingDetector != null) {
						levelCrossingDetector.Test(time, value);
						upCrossing = levelCrossingDetector.IsUpCrossing ? levelCrossingDetector.CrossingPoint : null;
						downCrossing = levelCrossingDetector.IsDownCrossing ? levelCrossingDetector.CrossingPoint : null;
					} else {
						upCrossing = null;
						downCrossing = null;
					}
					
					dataPoint = new double[] {
						time,
						value,
						level1,
						level2,
					};
					
					lock (dataBuffer.SyncRoot) {
						dataBuffer.Add(dataPoint);
					}
					
					if (!ReferenceEquals(upCrossing, null)) {
						lock (upBuffer.SyncRoot) {
							upBuffer.Add(upCrossing);
						}
					}
					
					if (!ReferenceEquals(downCrossing, null)) {
						lock (downBuffer.SyncRoot) {
							downBuffer.Add(downCrossing);
						}
					}
					
					
					time += timeIncrement;
				}
			}
			if (!isReady) return;
			UpdatePlots();
		}
		
		protected void UpdatePlots()
		{
			if (this.InvokeRequired) {
				MethodInvoker method = UpdatePlots;
				this.Invoke(method);
			} else {
				isReady = false;
				bool showLevel1 = (levelCrossingDetector != null) && levelCrossingDetector.IsActive && isLevelVisible; 
				bool showLevel2 = showLevel1 && (levelCrossingDetector.Hysteresis != 0.0);
				double tMin;
				double t;
				double x;
				double l1;
				double l2;
				DataPoint[] points;
				
				this.Series.SuspendUpdates();
				foreach (Series series in this.Series) {
					series.Points.SuspendUpdates();
				}
				
				lock (dataBuffer.SyncRoot) {
					for (int i = 0; i < dataBuffer.Count; i++) {
						t = dataBuffer[i][0];
						x = dataBuffer[i][1];
						l1 = dataBuffer[i][2];
						l2 = dataBuffer[i][3];
						
						points = new DataPoint[] {
							new DataPoint(t, x),
							Double.IsNaN(l1) ? null : new DataPoint(t, l1),
							Double.IsNaN(l2) ?  null : new DataPoint(t, l2),
						};
						
						
						for (int j = 0; j < points.Length; j++) {
							if (this.Series[j].Points.Count < dataBuffer.Count) {
								if ((points[j] != null) && (i >= this.Series[j].Points.Count)) this.Series[j].Points.Add(points[j]);
							} else {
								if (points[j] != null) this.Series[j].Points[i] = points[j];
							}
						}
					}
				}
				
				upCrossings.Points.Clear();
				downCrossings.Points.Clear();
				
				tMin = (this.Series[0].Points.Count > 0) ? this.Series[0].Points[0].XValue : 0.0;
				
				if (showLevel1) {
					lock (upBuffer.SyncRoot) {
						for (int i = 0; i < upBuffer.Count; i++) {
							if (upBuffer[i].X > tMin) {
								upCrossings.Points.AddXY(upBuffer[i].X, upBuffer[i].Y);
							}
						}
					}
					
					lock (downBuffer.SyncRoot) {
						for (int i = 0; i < downBuffer.Count; i++) {
							if (downBuffer[i].X > tMin) {
								downCrossings.Points.AddXY(downBuffer[i].X, downBuffer[i].Y);
							}
						}
					}
				}
				
				
				level1.Enabled = showLevel1;
				level2.Enabled = showLevel2;
				upCrossings.Enabled = showLevel1;
				downCrossings.Enabled = showLevel1;
				
				this.Series.ResumeUpdates();
				foreach (Series series in this.Series) {
					series.Points.ResumeUpdates();
				}
				
				this.ChartAreas[0].RecalculateAxesScale();
				this.Refresh();
				isReady = true;
			}
		}
	}
}
